@@ -16,6 +16,8 @@ import collections
 import gevent
 import tango.gevent
 
+import colorful
+
 
 Database = tango.Database
 Device = tango.gevent.DeviceProxy
@@ -322,6 +324,17 @@ def fnmatch_any(name, patterns, case_insensitive=False):
     return any(fnmatch.fnmatch(name, pattern) for pattern in patterns)
 
 
+def style(msg, fg=None, bg=None, bold=None, underlined=None, inversed=None):
+    st = []
+    bold and st.append("bold")
+    underlined and st.append("underlined")
+    inversed and st.append("inversed")
+    fg and st.append(fg)
+    bg and st.append("on_" + bg)
+    f = getattr(colorful, "_".join(st))
+    return f(msg)
+
+
 # ----- API -----
 
 
@@ -343,8 +356,11 @@ def iter_devices(
     )
     devices = (d for d in devices if fnmatch_any(d.klass, klass))
     devices = (d for d in devices if fnmatch_any(d.server, server))
-    devices = (d for d in devices if fnmatch_any(servers[d.server], host,
-                                                 case_insensitive=True))
+    devices = (
+        d
+        for d in devices
+        if fnmatch_any(servers[d.server], host, case_insensitive=True)
+    )
     return devices
 
 
@@ -358,6 +374,63 @@ def iter_servers(server=None, server_type=None, server_instance=None, db=None):
     return servs
 
 
+def server_tree(
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    compact=False,
+    server_only=False,
+    exclude_dserver=True,
+    db=None,
+):
+    import treelib
+
+    db_info = get_db_info(db=db)
+    db_name = db_info.name
+
+    tree = treelib.Tree()
+    db_node = tree.create_node(db_name)
+    servers = iter_servers(
+        server=server_name,
+        server_type=server_type,
+        server_instance=server_instance,
+        db=db,
+    )
+
+    if compact:
+        for serv in servers:
+            serv_inst_node = tree.create_node(serv.name, parent=db_node)
+            if not server_only:
+                for device in sorted(serv.devices):
+                    dlower = device.lower()
+                    if exclude_dserver and dlower.startswith("dserver/"):
+                        continue
+                    device_info = db_info.devices[dlower]
+                    text = "{} ({})".format(device, device_info.klass)
+                    tree.create_node(text, parent=serv_inst_node)
+    else:
+        # group servers by type
+        serv_map = collections.defaultdict(dict)
+        for serv in servers:
+            serv_map[serv.type][serv.instance] = serv
+
+        for serv_type in sorted(serv_map):
+            instances = serv_map[serv_type]
+            serv_type_node = tree.create_node(serv_type, parent=db_node)
+            for serv_inst in sorted(instances):
+                serv = instances[serv_inst]
+                serv_inst_node = tree.create_node(serv_inst, parent=serv_type_node)
+                if not server_only:
+                    for device in sorted(serv.devices):
+                        dlower = device.lower()
+                        if exclude_dserver and dlower.startswith("dserver/"):
+                            continue
+                        device_info = db_info.devices[dlower]
+                        text = "{} ({})".format(device, device_info.klass)
+                        tree.create_node(text, parent=serv_inst_node)
+    return tree
+
+
 def server_info(name, db=None):
     dserver = "dserver/" + name
     r = device_info(dserver, db=db)
@@ -579,6 +652,26 @@ def device_property_list(dev_name, db=None):
     return tuple(db.get_device_property_list(dev_name, "*"))
 
 
+def device_alias_list(dev_name, db=None):
+    db = get_db(db)
+    return tuple(db.get_device_alias_list(dev_name))
+
+
+def alias_from_device(dev_name, db=None):
+    db = get_db(db)
+    return db.get_alias_from_device(dev_name)
+
+
+def set_device_alias(dev_name, alias, db=None):
+    db = get_db(db)
+    db.put_device_alias(dev_name, alias)
+
+
+def remove_device_alias(alias, db=None):
+    db = get_db(db)
+    db.delete_device_alias(alias)
+
+
 def starter_names(db=None):
     db = get_db(db)
     return [s.split("/", 1)[-1] for s in db.get_server_list("Starter/*")]
@@ -995,3 +1088,60 @@ def starter_restart_servers(
             dry_run=dry_run,
             db=db,
         )
+
+
+def starter_tree(
+    starter_name=None,
+    server_name=None,
+    server_type=None,
+    server_instance=None,
+    all=False,
+    colored=True,
+    db=None,
+):
+
+    import treelib
+
+    db_info = get_db_info(db=db)
+    db_name = db_info.name
+
+    tree = treelib.Tree()
+    db_node = tree.create_node(db_name)
+    starters = list(iter_starters(starter_name=starter_name, db=db))
+    starters_servers_dict = starters_servers(*starters)
+    for starter in starters:
+        level_servers = collections.defaultdict(list)
+        servers = starters_servers_dict[starter]
+        fg = "red2" if servers is None else "green3"
+        name = str(style(starter.name, fg=fg)) if colored else starter.name
+        starter_node = tree.create_node(name, parent=db_node)
+        servers = servers.values() if servers else []
+        servers = iter_starter_servers(
+            servers, server_name, server_type, server_instance
+        )
+        for starter_server in servers:
+            level = level_servers[starter_server["level"]]
+            level.append(starter_server)
+        for level in sorted(level_servers):
+            if level == 0:
+                if not all:
+                    continue
+                name = "Not controlled"
+            else:
+                name = "level {}".format(level)
+            level_node = tree.create_node(name, parent=starter_node)
+            for starter_server in level_servers[level]:
+                state = starter_server["state"]
+                label = starter_server["name"]
+                if colored:
+                    if state == "ON":
+                        color = "green3"
+                    elif state == "FAULT":
+                        color = "red2"
+                    elif state == "MOVING":
+                        color = "royalBlue1"
+                    else:
+                        color = "magenta"
+                    label = str(style(label, fg=color))
+                tree.create_node(label, parent=level_node)
+    return tree
